#!/usr/bin/perl -w -- -d:ptkdb
#
# rsa_test.pl
#
# This script tests all classes and it's methods which are
# required by nabou for public-key crypto.
#
# This will be executed by 'make test' and should return:
#
#   11 of 11 tests run successful - OK
#
# If not, install the required module(s) and try again.
#
# By  accessing  this  software,  NABOU, you  are  duly informed
# of and agree to be  bound  by the  conditions  described below
# in this notice:
#
# This  software product,  NABOU,  is developed by Thomas Linden
# and  copyrighted  (C)  2000-2004  by  Thomas Linden,  with all
# rights reserved.
#
# There  is  no charge for NABOU software.  You can redistribute
# it and/or modify it under the terms of the GNU  General Public
# License, which is incorporated by reference herein.
#
# NABOU is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS,
# OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that
# the use of it will not infringe on any third party's intellec-
# tual property rights.
#
# You  should  have received a copy of the  GNU  General  Public
# License  along with NABOU.   Copies can also be obtained from:
#
#   http://www.gnu.org/licenses/gpl.txt
#
# or by writing to:
#
#   Free Software Foundation, Inc.
#   59 Temple Place, Suite 330
#   Boston, MA 02111-1307
#   USA
#
# Or contact:
#
#  "Thomas Linden" <tom@daemon.de>
#
# Find more details about NABOU on its website:
#
#  http://www.daemon.de/Nabou
#


use strict;

BEGIN {
print "Loading modules:\n\n";
print "use Crypt::OpenSSL::RSA\n";
use Crypt::OpenSSL::RSA;

print "use Crypt::CBC\n";
use Crypt::CBC;

print "use MIME::Base64\n";
use MIME::Base64;

print "use Crypt::Primes qw(maurer)\n";
use Crypt::Primes qw(maurer);
}

my $expect = 11;
my $suck = 4;

my $plain = "  This is the clear text message.
  This is the clear text message.";



print "\nGenerating prime 512 bit\n";
my $prime = maurer (Size => 512, Verbosity => 1);
$suck++;

print "\nGenerating 512 bit keys\n";
my $rsa_gen = Crypt::OpenSSL::RSA->generate_key(512, $prime);
$suck++;
my $private = $rsa_gen->get_private_key_string();
my $public  = $rsa_gen->get_public_key_string();
$suck++;

my $rsa_pub  = Crypt::OpenSSL::RSA->new_public_key($public);
my $rsa_priv = Crypt::OpenSSL::RSA->new_private_key($private);

$private =~ s/^.*(-----BEGIN.*KEY-----).*$/$1\n/s;
$public  =~ s/^.*(-----BEGIN.*KEY-----).*$/$1\n/s;


print "Generated keys: \n" . $private . "\n" . $public . "\n";


print "Encrypting private key using pass-phrase \"foo bar\" with CBC::Blowfish\n";
my $cypher = new Crypt::CBC("foo bar", 'Blowfish');
$suck++;
my $cr_private = $private;
$cr_private  =~ s/^.*(-----BEGIN RSA PRIVATE KEY-----)\n(.*)\n(-----END RSA PRIVATE KEY-----).*$/$2/s;
my $private_cypher = encode_base64($cypher->encrypt($cr_private));
$suck++;

$cr_private = "-----BEGIN ENCRYPTED RSA PRIVATE KEY-----\n"
           . $private_cypher
           . "-----END ENCRYPTED RSA PRIVATE KEY-----\n";

print "Encrypted private key: \n" . $cr_private . "\n";


print "Generating signature for the following text:\n$plain\n";
my $sig = $rsa_priv->sign($plain);
print "Signature for plain text (base64 encoded):\n";
print  MIME::Base64::encode_base64($sig);
$suck++;

print "Verifying signature:\n";
if ($rsa_pub->verify($plain, $sig)) {
  print "Signature valid.\n";
  $suck ++;
}
else {
  print "Signature invalid!\n";
}


print "$suck of $expect tests run successful - ";
if ($suck == $expect) {
  print "OK\n";
}
else {
  print "BAD\n";
}

exit;
